#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>

static void
run_dhcpcd(const char *ifname)
{
	pid_t pid;

	pid = fork();

	if (pid == -1)
	{
		perror("fork");
		return;
	}
	else if (pid > 0)
	{
		/* we are the parent */
		return;
	}

	if (execlp("dhcpcd", "dhcpcd", "--noipv4ll", ifname, NULL) == -1)
	{
		perror("dhcpcd");
	}

	_exit(EXIT_FAILURE);
}

void
run_dhcp(const char *ifname)
{
	char path[128], buf[16];
	struct stat st;
	int fd, pid;

	memset(&st, 0, sizeof(struct stat));
	snprintf(path, sizeof(path), "/run/dhcpcd-%s.pid", ifname);

	if (stat(path, &st) != 0)
	{
		perror(path);
		fprintf(stderr, "[-] DHCP client not running on %s\n", ifname);
		run_dhcpcd(ifname);
		return;
	}

	fd = open(path, 0);
	if (fd == -1)
	{
		perror(path);
		fprintf(stderr, "[!] Failed to open DHCP client PID file\n");
		return;
	}

	memset(buf, 0, sizeof(buf));
	if (read(fd, buf, sizeof(buf) - 1) == -1)
	{
		perror(path);
		fprintf(stderr, "[!] Failed to read DHCP client PID file\n");
		close(fd);
		return;
	}
	close(fd);

	pid = atoi(buf);
	if (pid == 0)
	{
		fprintf(stderr, "[!] Failed to parse PID: %s\n", buf);
		return;
	}

	snprintf(path, sizeof(path), "/proc/%d", pid);
	if (stat(path, &st) != 0)
	{
		perror(path);
		fprintf(stderr, "[!] DHCP client no longer running\n");
		run_dhcpcd(ifname);
		return;
	}

	printf("[+] DHCP client running on %s\n", ifname);
}

void
stop_dhcp(const char *ifname)
{
	char path[128], buf[16];
	struct stat st;
	int fd, pid;

	memset(&st, 0, sizeof(struct stat));
	snprintf(path, sizeof(path), "/run/dhcpcd-%s.pid", ifname);

	if (stat(path, &st) != 0)
	{
		fprintf(stderr, "[+] DHCP client not running on %s\n", ifname);
		return;
	}

	fd = open(path, 0);
	if (fd == -1)
	{
		perror(path);
		fprintf(stderr, "[!] Failed to open DHCP client PID file\n");
		return;
	}

	memset(buf, 0, sizeof(buf));
	if (read(fd, buf, sizeof(buf) - 1) == -1)
	{
		perror(path);
		fprintf(stderr, "[!] Failed to read DHCP client PID file\n");
		close(fd);
		return;
	}
	close(fd);

	pid = atoi(buf);
	if (pid == 0)
	{
		fprintf(stderr, "[!] Failed to parse PID: %s\n", buf);
		return;
	}

	snprintf(path, sizeof(path), "/proc/%d", pid);
	if (stat(path, &st) != 0)
	{
		perror(path);
		fprintf(stderr, "[!] DHCP client no longer running\n");
		return;
	}

	printf("[-] DHCP client running on %s, terminating\n", ifname);
	kill(pid, SIGTERM);
}
